@@ -9,18 +9,20 @@ import (
 	"expvar"
 	"flag"
 	"fmt"
-	"html/template"
 	"io/ioutil"
 	"math/rand"
 	"net"
 	"os"
+	"os/signal"
 	"path/filepath"
 	"strconv"
+	"syscall"
 	"time"
 
 	"blitiri.com.ar/go/chasquid/internal/config"
 	"blitiri.com.ar/go/chasquid/internal/courier"
 	"blitiri.com.ar/go/chasquid/internal/dovecot"
+	"blitiri.com.ar/go/chasquid/internal/expvarom"
 	"blitiri.com.ar/go/chasquid/internal/maillog"
 	"blitiri.com.ar/go/chasquid/internal/normalize"
 	"blitiri.com.ar/go/chasquid/internal/smtpsrv"
@@ -28,15 +30,14 @@ import (
 	"blitiri.com.ar/go/chasquid/internal/userdb"
 	"blitiri.com.ar/go/log"
 	"blitiri.com.ar/go/systemd"
-
-	"net/http"
-	_ "net/http/pprof"
 )
 
 // Command-line flags.
 var (
 	configDir = flag.String("config_dir", "/etc/chasquid",
 		"configuration directory")
+	configOverrides = flag.String("config_overrides", "",
+		"override configuration values (in text protobuf format)")
 	showVer = flag.Bool("version", false, "show version and exit")
 )
 
@@ -52,7 +53,8 @@ var (
 
 	sourceDate      time.Time
 	sourceDateVar   = expvar.NewString("chasquid/sourceDateStr")
-	sourceDateTsVar = expvar.NewInt("chasquid/sourceDateTimestamp")
+	sourceDateTsVar = expvarom.NewInt("chasquid/sourceDateTimestamp",
+		"timestamp when the binary was built, in seconds since epoch")
 )
 
 func main() {
@@ -70,9 +72,9 @@ func main() {
 	// Seed the PRNG, just to prevent for it to be totally predictable.
 	rand.Seed(time.Now().UnixNano())
 
-	conf, err := config.Load(*configDir + "/chasquid.conf")
+	conf, err := config.Load(*configDir+"/chasquid.conf", *configOverrides)
 	if err != nil {
-		log.Fatalf("Error reading config: %v", err)
+		log.Fatalf("Error loading config: %v", err)
 	}
 	config.LogConfig(conf)
 
@@ -81,12 +83,17 @@ func main() {
 	// It also can be useful in unusual environments and for testing purposes,
 	// where paths inside the configuration itself could be relative, and this
 	// fixes the point of reference.
-	os.Chdir(*configDir)
+	err = os.Chdir(*configDir)
+	if err != nil {
+		log.Fatalf("Error changing to config dir %q: %v", *configDir, err)
+	}
 
 	initMailLog(conf.MailLogPath)
 
+	go signalHandler()
+
 	if conf.MonitoringAddress != "" {
-		launchMonitoringServer(conf.MonitoringAddress)
+		launchMonitoringServer(conf)
 	}
 
 	s := smtpsrv.NewServer()
@@ -157,7 +164,11 @@ func main() {
 		Args:    conf.MailDeliveryAgentArgs,
 		Timeout: 30 * time.Second,
 	}
-	remoteC := &courier.SMTP{Dinfo: dinfo, STSCache: stsCache}
+	remoteC := &courier.SMTP{
+		HelloDomain: conf.Hostname,
+		Dinfo:       dinfo,
+		STSCache:    stsCache,
+	}
 	s.InitQueue(conf.DataDir+"/queue", localC, remoteC)
 
 	// Load the addresses and listeners.
@@ -203,13 +214,16 @@ func loadAddresses(srv *smtpsrv.Server, addrs []string, ls []net.Listener, mode
 func initMailLog(path string) {
 	var err error
 
-	if path == "<syslog>" {
+	switch path {
+	case "<syslog>":
 		maillog.Default, err = maillog.NewSyslog()
-	} else {
-		os.MkdirAll(filepath.Dir(path), 0775)
-		var f *os.File
-		f, err = os.OpenFile(path, os.O_WRONLY|os.O_APPEND|os.O_CREATE, 0664)
-		maillog.Default = maillog.New(f)
+	case "<stdout>":
+		maillog.Default = maillog.New(os.Stdout)
+	case "<stderr>":
+		maillog.Default = maillog.New(os.Stderr)
+	default:
+		_ = os.MkdirAll(filepath.Dir(path), 0775)
+		maillog.Default, err = maillog.NewFile(path)
 	}
 
 	if err != nil {
@@ -217,6 +231,32 @@ func initMailLog(path string) {
 	}
 }
 
+func signalHandler() {
+	var err error
+
+	signals := make(chan os.Signal, 1)
+	signal.Notify(signals, syscall.SIGHUP)
+
+	for {
+		switch sig := <-signals; sig {
+		case syscall.SIGHUP:
+			// SIGHUP triggers a reopen of the log files. This is used for log
+			// rotation.
+			err = log.Default.Reopen()
+			if err != nil {
+				log.Fatalf("Error reopening log: %v", err)
+			}
+
+			err = maillog.Default.Reopen()
+			if err != nil {
+				log.Fatalf("Error reopening maillog: %v", err)
+			}
+		default:
+			log.Errorf("Unexpected signal %v", sig)
+		}
+	}
+}
+
 // Helper to load a single domain configuration into the server.
 func loadDomain(name, dir string, s *smtpsrv.Server) {
 	log.Infof("  %s", name)
@@ -280,94 +320,3 @@ func parseVersionInfo() {
 	sourceDateVar.Set(sourceDate.Format("2006-01-02 15:04:05 -0700"))
 	sourceDateTsVar.Set(sdts)
 }
-
-func launchMonitoringServer(addr string) {
-	log.Infof("Monitoring HTTP server listening on %s", addr)
-
-	indexData := struct {
-		Version    string
-		SourceDate time.Time
-		StartTime  time.Time
-	}{
-		Version:    version,
-		SourceDate: sourceDate,
-		StartTime:  time.Now(),
-	}
-
-	http.HandleFunc("/", func(w http.ResponseWriter, r *http.Request) {
-		if r.URL.Path != "/" {
-			http.NotFound(w, r)
-			return
-		}
-		if err := monitoringHTMLIndex.Execute(w, indexData); err != nil {
-			log.Infof("monitoring handler error: %v", err)
-		}
-	})
-
-	flags := dumpFlags()
-	http.HandleFunc("/debug/flags", func(w http.ResponseWriter, r *http.Request) {
-		w.Write([]byte(flags))
-	})
-
-	go http.ListenAndServe(addr, nil)
-}
-
-// Static index for the monitoring website.
-var monitoringHTMLIndex = template.Must(template.New("index").Funcs(
-	template.FuncMap{"since": time.Since}).Parse(
-	`<!DOCTYPE html>
-<html>
-  <head>
-    <title>chasquid monitoring</title>
-  </head>
-  <body>
-    <h1>chasquid monitoring</h1>
-
-	chasquid {{.Version}}<br>
-	source date {{.SourceDate.Format "2006-01-02 15:04:05 -0700"}}<p>
-
-	started {{.StartTime.Format "Mon, 2006-01-02 15:04:05 -0700"}}<br>
-	up since {{.StartTime | since}}<p>
-
-    <ul>
-      <li><a href="/debug/queue">queue</a>
-      <li><a href="/debug/vars">exported variables</a>
-	       <small><a href="https://golang.org/pkg/expvar/">(ref)</a></small>
-	  <li>traces <small><a href="https://godoc.org/golang.org/x/net/trace">
-            (ref)</a></small>
-        <ul>
-          <li><a href="/debug/requests?exp=1">requests (short-lived)</a>
-          <li><a href="/debug/events?exp=1">events (long-lived)</a>
-        </ul>
-      <li><a href="/debug/flags">flags</a>
-      <li><a href="/debug/pprof">pprof</a>
-          <small><a href="https://golang.org/pkg/net/http/pprof/">
-            (ref)</a></small>
-        <ul>
-          <li><a href="/debug/pprof/goroutine?debug=1">goroutines</a>
-        </ul>
-    </ul>
-  </body>
-</html>
-`))
-
-// dumpFlags to a string, for troubleshooting purposes.
-func dumpFlags() string {
-	s := ""
-	visited := make(map[string]bool)
-
-	// Print set flags first, then the rest.
-	flag.Visit(func(f *flag.Flag) {
-		s += fmt.Sprintf("-%s=%s\n", f.Name, f.Value.String())
-		visited[f.Name] = true
-	})
-
-	s += "\n"
-	flag.VisitAll(func(f *flag.Flag) {
-		if !visited[f.Name] {
-			s += fmt.Sprintf("-%s=%s\n", f.Name, f.Value.String())
-		}
-	})
-
-	return s
-}
