@@ -20,9 +20,9 @@ import (
 	"blitiri.com.ar/go/chasquid/internal/envelope"
 	"blitiri.com.ar/go/chasquid/internal/normalize"
 	"blitiri.com.ar/go/chasquid/internal/userdb"
+	"google.golang.org/protobuf/encoding/prototext"
 
 	"github.com/docopt/docopt-go"
-	"github.com/golang/protobuf/proto"
 	"golang.org/x/crypto/ssh/terminal"
 )
 
@@ -51,7 +51,7 @@ var (
 )
 
 func main() {
-	args, _ = docopt.Parse(usage, nil, true, "", false)
+	args, _ = docopt.ParseDoc(usage)
 
 	// Load globals.
 	if d, ok := args["--configdir"].(string); ok {
@@ -100,7 +100,10 @@ func userDBFromArgs(create bool) (string, string, *userdb.DB) {
 	if err != nil {
 		if create && os.IsNotExist(err) {
 			fmt.Println("Creating database")
-			os.MkdirAll(filepath.Dir(userDBForDomain(domain)), 0755)
+			err = os.MkdirAll(filepath.Dir(userDBForDomain(domain)), 0755)
+			if err != nil {
+				Fatalf("Error creating database dir: %v", err)
+			}
 		} else {
 			Fatalf("Error loading database: %v", err)
 		}
@@ -201,11 +204,11 @@ func userRemove() {
 
 // chasquid-util aliases-resolve <address>
 func aliasesResolve() {
-	conf, err := config.Load(configDir + "/chasquid.conf")
+	conf, err := config.Load(configDir+"/chasquid.conf", "")
 	if err != nil {
-		Fatalf("Error reading config")
+		Fatalf("Error loading config: %v", err)
 	}
-	os.Chdir(configDir)
+	_ = os.Chdir(configDir)
 
 	r := aliases.NewResolver()
 	r.SuffixSep = conf.SuffixSeparators
@@ -245,21 +248,21 @@ func aliasesResolve() {
 
 // chasquid-util print-config
 func printConfig() {
-	conf, err := config.Load(configDir + "/chasquid.conf")
+	conf, err := config.Load(configDir+"/chasquid.conf", "")
 	if err != nil {
-		Fatalf("Error reading config")
+		Fatalf("Error loading config: %v", err)
 	}
 
-	fmt.Println(proto.MarshalTextString(conf))
+	fmt.Println(prototext.Format(conf))
 }
 
 // chasquid-util domaininfo-remove <domain>
 func domaininfoRemove() {
 	domain := args["<domain>"].(string)
 
-	conf, err := config.Load(configDir + "/chasquid.conf")
+	conf, err := config.Load(configDir+"/chasquid.conf", "")
 	if err != nil {
-		Fatalf("Error reading config")
+		Fatalf("Error loading config: %v", err)
 	}
 
 	// File for the corresponding domain.
@@ -287,11 +290,11 @@ func aliasesAdd() {
 		Fatalf("Domain doesn't exist")
 	}
 
-	conf, err := config.Load(configDir + "/chasquid.conf")
+	conf, err := config.Load(configDir+"/chasquid.conf", "")
 	if err != nil {
-		Fatalf("Error reading config")
+		Fatalf("Error loading config: %v", err)
 	}
-	os.Chdir(configDir)
+	_ = os.Chdir(configDir)
 
 	// Setup alias resolver.
 	r := aliases.NewResolver()
