@@ -5,7 +5,6 @@ import (
 	"bytes"
 	"context"
 	"crypto/tls"
-	"expvar"
 	"flag"
 	"fmt"
 	"io"
@@ -25,6 +24,7 @@ import (
 	"blitiri.com.ar/go/chasquid/internal/auth"
 	"blitiri.com.ar/go/chasquid/internal/domaininfo"
 	"blitiri.com.ar/go/chasquid/internal/envelope"
+	"blitiri.com.ar/go/chasquid/internal/expvarom"
 	"blitiri.com.ar/go/chasquid/internal/maillog"
 	"blitiri.com.ar/go/chasquid/internal/normalize"
 	"blitiri.com.ar/go/chasquid/internal/queue"
@@ -36,13 +36,20 @@ import (
 
 // Exported variables.
 var (
-	commandCount      = expvar.NewMap("chasquid/smtpIn/commandCount")
-	responseCodeCount = expvar.NewMap("chasquid/smtpIn/responseCodeCount")
-	spfResultCount    = expvar.NewMap("chasquid/smtpIn/spfResultCount")
-	loopsDetected     = expvar.NewInt("chasquid/smtpIn/loopsDetected")
-	tlsCount          = expvar.NewMap("chasquid/smtpIn/tlsCount")
-	slcResults        = expvar.NewMap("chasquid/smtpIn/securityLevelChecks")
-	hookResults       = expvar.NewMap("chasquid/smtpIn/hookResults")
+	commandCount = expvarom.NewMap("chasquid/smtpIn/commandCount",
+		"command", "count of SMTP commands received, by command")
+	responseCodeCount = expvarom.NewMap("chasquid/smtpIn/responseCodeCount",
+		"code", "response codes returned to SMTP commands")
+	spfResultCount = expvarom.NewMap("chasquid/smtpIn/spfResultCount",
+		"result", "SPF result count")
+	loopsDetected = expvarom.NewInt("chasquid/smtpIn/loopsDetected",
+		"count of loops detected")
+	tlsCount = expvarom.NewMap("chasquid/smtpIn/tlsCount",
+		"status", "count of TLS usage in incoming connections")
+	slcResults = expvarom.NewMap("chasquid/smtpIn/securityLevelChecks",
+		"result", "incoming security level check results")
+	hookResults = expvarom.NewMap("chasquid/smtpIn/hookResults",
+		"result", "count of hook invocations, by result")
 )
 
 var (
@@ -174,7 +181,12 @@ func (c *Conn) Handle() {
 	if tc, ok := c.conn.(*tls.Conn); ok {
 		// For TLS connections, complete the handshake and get the state, so
 		// it can be used when we say hello below.
-		tc.Handshake()
+		err := tc.Handshake()
+		if err != nil {
+			c.tr.Errorf("error completing TLS handshake: %v", err)
+			return
+		}
+
 		cstate := tc.ConnectionState()
 		c.tlsConnState = &cstate
 		if name := c.tlsConnState.ServerName; name != "" {
@@ -245,7 +257,7 @@ loop:
 		case "AUTH":
 			code, msg = c.AUTH(params)
 		case "QUIT":
-			c.writeResponse(221, "2.0.0 Be seeing you...")
+			_ = c.writeResponse(221, "2.0.0 Be seeing you...")
 			break loop
 		default:
 			// Sanitize it a bit to avoid filling the logs and events with
@@ -267,7 +279,7 @@ loop:
 				if errCount > 10 {
 					// https://tools.ietf.org/html/rfc5321#section-4.3.2
 					c.tr.Errorf("too many errors, breaking connection")
-					c.writeResponse(421, "4.5.0 Too many errors, bye")
+					_ = c.writeResponse(421, "4.5.0 Too many errors, bye")
 					break
 				}
 			}
@@ -601,7 +613,7 @@ func (c *Conn) DATA(params string) (code int, msg string) {
 			// the user keeps sending as commands, and that's a security
 			// issue.
 			readUntilDot(c.reader)
-			return 552, fmt.Sprintf("5.3.4 Message too big")
+			return 552, "5.3.4 Message too big"
 		}
 		return 554, fmt.Sprintf("5.4.0 Error reading DATA: %v", err)
 	}
@@ -999,9 +1011,12 @@ func (c *Conn) AUTH(params string) (code int, msg string) {
 		return 501, fmt.Sprintf("5.5.2 Error decoding AUTH response: %v", err)
 	}
 
+	// https://tools.ietf.org/html/rfc4954#section-6
 	authOk, err := c.authr.Authenticate(user, domain, passwd)
 	if err != nil {
 		c.tr.Errorf("error authenticating %q@%q: %v", user, domain, err)
+		maillog.Auth(c.conn.RemoteAddr(), user+"@"+domain, false)
+		return 454, "4.7.0 Temporary authentication failure"
 	}
 	if authOk {
 		c.authUser = user
@@ -1085,9 +1100,9 @@ func (c *Conn) writeResponse(code int, msg string) error {
 	return writeResponse(c.writer, code, msg)
 }
 
-func (c *Conn) printfLine(format string, args ...interface{}) error {
+func (c *Conn) printfLine(format string, args ...interface{}) {
 	fmt.Fprintf(c.writer, format+"\r\n", args...)
-	return c.writer.Flush()
+	c.writer.Flush()
 }
 
 // writeResponse writes a multi-line response to the given writer.
