@@ -147,8 +147,12 @@ func (s *Server) InitDomainInfo(dir string) *domaininfo.DB {
 
 // InitQueue initializes the queue.
 func (s *Server) InitQueue(path string, localC, remoteC courier.Courier) {
-	q := queue.New(path, s.localDomains, s.aliasesR, localC, remoteC)
-	err := q.Load()
+	q, err := queue.New(path, s.localDomains, s.aliasesR, localC, remoteC)
+	if err != nil {
+		log.Fatalf("Error initializing queue: %v:", err)
+	}
+
+	err = q.Load()
 	if err != nil {
 		log.Fatalf("Error loading queue: %v", err)
 	}
@@ -156,7 +160,7 @@ func (s *Server) InitQueue(path string, localC, remoteC courier.Courier) {
 
 	http.HandleFunc("/debug/queue",
 		func(w http.ResponseWriter, r *http.Request) {
-			w.Write([]byte(q.DumpString()))
+			_, _ = w.Write([]byte(q.DumpString()))
 		})
 }
 
@@ -198,6 +202,8 @@ func (s *Server) ListenAndServe() {
 
 	// At this point the TLS config should be done, build the
 	// name->certificate map (used by the TLS library for SNI).
+	// TODO: Once we support only Go >= 1.14, we can drop this, as it is no
+	// longer necessary.
 	s.tlsConfig.BuildNameToCertificate()
 
 	go s.periodicallyReload()
