@@ -34,14 +34,34 @@ run_msmtp aliasB@srv-B < content
 
 # Get some of the debugging pages, for troubleshooting, and to make sure they
 # work reasonably well.
-wget -q -o /dev/null -O .data-A/dbg-root http://localhost:1099/ \
+function fetch() {
+	wget -q -o /dev/null -O $2 $1
+}
+
+function linesgt10() {
+	[ $( cat $1 | wc -l ) -gt 10 ]
+}
+
+fetch http://localhost:1099/ .data-A/dbg-root \
+	&& linesgt10 .data-A/dbg-root \
 	|| fail "failed to fetch /"
-wget -q -o /dev/null -O .data-A/dbg-flags http://localhost:1099/debug/flags \
+fetch http://localhost:1099/debug/flags .data-A/dbg-flags \
+	&& linesgt10 .data-A/dbg-flags \
 	|| fail "failed to fetch /debug/flags"
-wget -q -o /dev/null -O .data-A/dbg-queue http://localhost:1099/debug/queue \
+fetch http://localhost:1099/debug/queue .data-A/dbg-queue \
 	|| fail "failed to fetch /debug/queue"
-wget -q -o /dev/null -O .data-A/dbg-root http://localhost:1099/404 \
+fetch http://localhost:1099/debug/config .data-A/dbg-config \
+	&& linesgt10 .data-A/dbg-config \
+	|| fail "failed to fetch /debug/config"
+fetch http://localhost:1099/404 .data-A/dbg-404 \
 	&& fail "fetch /404 worked, should have failed"
+fetch http://localhost:1099/metrics .data-A/metrics \
+	&& linesgt10 .data-A/metrics \
+	|| fail "failed to fetch /metrics"
+
+# Quick sanity-check of the /metrics page, just in case.
+grep -q '^chasquid_queue_itemsWritten [0-9]\+$' .data-A/metrics \
+	|| fail "A /metrics is missing the chasquid_queue_itemsWritten counter"
 
 # Wait until one of them has noticed and stopped the loop.
 while sleep 0.1; do
@@ -57,7 +77,7 @@ done
 # Test that A has outgoing domaininfo for srv-b.
 # This is unrelated to the loop itself, but serves as an end-to-end
 # verification that outgoing domaininfo works.
-if ! grep -q "outgoing_sec_level: TLS_INSECURE" ".data-A/domaininfo/s:srv-b";
+if ! grep -q 'outgoing_sec_level:\s*TLS_INSECURE' ".data-A/domaininfo/s:srv-b";
 then
 	fail "A is missing the domaininfo for srv-b"
 fi
